/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file Application.h
/// This source file is part of Gpu Quadratic Curve Render Test.
/// The file holds the Application class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#ifndef _APPLICATION_H_
#define _APPLICATION_H_

#include "Runner.h"
#include "Window.h"

namespace GpuQuadraticCurveRender
{

/// @brief It's the main point entry of the test. Creates the
/// test runner and then, creates the main window
/// @see GlutWrapper::Window
/// @see GpuQuadraticCurveRenderTestRunner
class Application
{
public:
	/// Constructor
	/// @param argc number of application arguments (for glut initialization)
	/// @param argv application arguments (for glut initialization)
	/// @param positionX render window position X coordinate
	/// @param positionY render window position Y coordinate
	/// @param width render window height
	/// @param windowName render window name
	Application(int argc, char** argv, int positionX, int positionY, 
					 int width, int height, const std::string& windowName);
	
	/// Destructor
	~Application(void);

private:
	/// Runner of constrained delaunay test
	Runner m_runner;

	/// Render window
	GlutWrapper::Window m_window;

	/// @note hide copy constructor from interface user
	Application(const Application&);

	/// @note hide assignment operator from interface user
	Application& operator=(const Application&);
};

}

#endif