/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file IKeyboardListener.h
/// This source file is part of Gpu Quadratic Curve Render test.
/// The file holds the IKeyboardListener class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#ifndef _I_KEYBOARD_LISTENER_H_
#define _I_KEYBOARD_LISTENER_H_

namespace GlutWrapper
{

enum Key
{
	BackSpace = 8,
	Enter = 13,
	Escape = 27,
	Delete = 127
};

/// @brief Keyboard listener interface. It processes the 
/// events of the keyboard.
class IKeyboardListener
{
public:
	/// @brief It receives a key pressed event.
	/// @param key the key pressed
	virtual void ProcessNormalKeys(unsigned char key) = 0;

protected:
	/// Contructor hidden from user
	IKeyboardListener(void){};

	/// Destructor hidden from user
	virtual ~IKeyboardListener(void) = 0 {};

private:
	/// @note hide copy constructor from interface user
	IKeyboardListener(const IKeyboardListener&);

	/// @note hide assignment operator from interface user
	IKeyboardListener& operator=(const IKeyboardListener&);
};

}

#endif