/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file IMenuListener.h
/// This source file is part of Gpu Quadratic Curve Render test.
/// The file holds the IMenuListener class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#ifndef _I_MENU_LISTENER_H_
#define _I_MENU_LISTENER_H_

namespace GlutWrapper
{

/// @brief Right mouse button menu listener interface. It processes
/// the mouse right button menu events.
class IMenuListener
{
public:
	/// @brief It's called when a right mouse button menu event
	/// is fired.
	/// @param option the option related to the event
	virtual void ProcessMenuEvents(int option) = 0;

protected:
	/// Contructor hidden from user
	IMenuListener(void){};

	/// Destructor hidden from user
	virtual ~IMenuListener(void) = 0 {};

private:
	/// @note hide copy constructor from interface user
	IMenuListener(const IMenuListener&);

	/// @note hide assignment operator from interface user
	IMenuListener& operator=(const IMenuListener&);
};
}

#endif