/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file OpenglGpuProgram.h
/// This source file is part of Gpu Quadratic Curve Render Test.
/// The file holds the OpenglGpuProgram class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)

#ifndef _OPENGL_GPU_PROGRAM_H
#define _OPENGL_GPU_PROGRAM_H

#include "OpenglGpuShader.h"

#include "GL\glew.h"
#include <string>

namespace GpuQuadraticCurveRender
{

/// @brief Opengl GLSL program. It links and compiles a 
/// vertex shader and a pixel shader
class OpenglGpuProgram
{
public:
	/// Constructor
	/// @param vertexShader path to vertex shader file
	/// @param pixelShader path to pixel shader file
	OpenglGpuProgram(const std::wstring& vertexShader, 
					 const std::wstring& pixelShader);
	
	/// Destructor
	~OpenglGpuProgram(void);

	/// @brief Gets gl program id
	/// @return gl program id
	GLint getGlProgramId(void) const;

private:
	/// Vertex shader
	OpenglGpuShader m_vertexShader;

	/// Pixel shader
	OpenglGpuShader m_pixelShader;

	/// Opengl program id
	GLint m_glProgram;

	/// @note hide copy constructor from interface user
	OpenglGpuProgram(const OpenglGpuProgram&);

	/// @note hide assignment operator from interface user
	OpenglGpuProgram& operator=(const OpenglGpuProgram&);

	/// @brief Creastes a program from a vertex shader file
	/// and a pixel shader file
	/// @param vertexShader vertex shader file path
	/// @param pixelShader pixel shader file path
	GLint CreateProgram(const OpenglGpuShader& vertexShader, 
						const OpenglGpuShader& pixelShader);
};

}

#endif