/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file OpenglGpuShader.h
/// This source file is part of Gpu Quadratic Curve Render Test.
/// The file holds the OpenglGpuShader class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)

#ifndef _OPENGL_GPU_SHADER_H_
#define _OPENGL_GPU_SHADER_H_

#include "GL\glew.h"
#include <string>

namespace GpuQuadraticCurveRender
{

enum GpuShaderStage
{
	Vertex,
	Pixel
};

/// @brief Compiles a glsl shader.
class OpenglGpuShader
{
public:
	/// Constructor
	/// @param shaderFile shader path file
	/// @param stage Vertex or pixel stage
	OpenglGpuShader(const std::wstring& shaderFile, GpuShaderStage stage);

	/// Destructor
	~OpenglGpuShader(void);

	/// @brief Gets gl shader id
	/// @return gl shader id
	GLint GetGlShaderId(void) const;

protected:
	/// Opengl shader id
	GLint m_glShader;

	/// @brief Compiles a shader file
	/// @param shaderFile shader path file
	/// @param stage Vertex or pixel stage
	GLint LoadShaderFromFile(const std::wstring& shaderFile, GpuShaderStage stage);

	/// @brief Reads source shader file
	/// @param shaderFile shader path file
	/// @param source shader source
	void ReadSourceFile(const std::wstring& shaderFile, std::string& source);

	/// @note hide copy constructor from interface user
	OpenglGpuShader(const OpenglGpuShader& shader);

	/// @note hide assignment operator from interface user
	OpenglGpuShader& operator=(const OpenglGpuShader& shader);
};

}

#endif