/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file UnicodeString.cpp
/// This source file is part of Gpu Quadratic Curve Render test.
/// The file holds unicode string functions implementations.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)

#include "UnicodeString.h"

#include <windows.h>

#include "Win32LastError.h"

std::string GpuQuadraticCurveRender::ConvertUTF16ToUTF8(const std::wstring& utf16String)
{
	/// Getting the required size
	int utf8StringSize = WideCharToMultiByte(CP_UTF8, 0, utf16String.c_str(), utf16String.size(), 0, 0, 0, 0);
	if (utf8StringSize == 0)
	{
		GpuQuadraticCurveRender::Win32LastError lastError;
		throw lastError.ToString();
	}

	/// Converting
	char* utf8CharArray = new char[utf8StringSize + 1];
	if (FAILED(WideCharToMultiByte(CP_UTF8, 0, utf16String.c_str(), utf16String.size(), utf8CharArray, utf8StringSize, 0, 0)))
	{
		delete[] utf8CharArray;
		GpuQuadraticCurveRender::Win32LastError lastError;
		throw lastError.ToString();
	}
	
	utf8CharArray[utf8StringSize] = '\0';

	std::string utf8String(utf8CharArray);
	
	delete[] utf8CharArray;

	return utf8String;
}

std::wstring GpuQuadraticCurveRender::ConvertUTF8ToUTF16(const std::string& utf8String)
{
	/// Getting the required size
	int utf16StringSize = MultiByteToWideChar(CP_UTF8, 0, utf8String.c_str(), utf8String.size(), 0, 0);
	if (utf16StringSize == 0)
	{
		GpuQuadraticCurveRender::Win32LastError lastError;
		throw lastError.ToString();
	}

	/// Converting
	wchar_t* utf16CharArray = new wchar_t[utf16StringSize + 1];
	if (FAILED(MultiByteToWideChar(CP_UTF8, 0, utf8String.c_str(), utf8String.size(), utf16CharArray, utf16StringSize)))
	{
		delete[] utf16CharArray;
		GpuQuadraticCurveRender::Win32LastError lastError;
		throw lastError.ToString();
	}

	utf16CharArray[utf16StringSize] = L'\0';

	std::wstring utf16String(utf16CharArray);
	
	delete[] utf16CharArray;

	return utf16String;
}
