/// Copyright (c) 2009 Ruben Penalva. All rights reserved.
/// Redistribution and use in source and binary forms, with or without modification, 
/// are permitted provided that the following conditions are met:
///    1. Redistributions of source code must retain the above copyright notice, this 
///    list of conditions and the following disclaimer.
///    2. Redistributions in binary form must reproduce the above copyright notice,
///    this list of conditions and the following disclaimer in the documentation and/or
///    other materials provided with the distribution.
///    3. The name of the author may not be used to endorse or promote products derived
///    from this software without specific prior written permission.
///
/// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
/// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
/// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, 
/// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
/// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
/// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
/// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

/// @file Win32LastError.h
/// This source file is part of Gpu Quadratic Curve Render Test.
/// The file holds the Win32LastError class interface.
/// @author Ruben Penalva (ruben.penalva@rpenalva.com)
#ifndef _WIN32_LAST_ERROR_H_
#define _WIN32_LAST_ERROR_H_

#include <string>

namespace GpuQuadraticCurveRender
{

/// @brief Retrive the last error code of windows and
/// format it to a string.
class Win32LastError
{
public:
	/// Constructor. Retrieve last error code.
	Win32LastError(void);

	/// Destructor.
	~Win32LastError(void);

	/// @brief Formatted error string.
	/// @return error string.
	const std::wstring& ToString(void) const;

private:
	/// Formatted error string
	std::wstring m_stringError;

	/// @note hide copy constructor from interface user
	Win32LastError(const Win32LastError&);

	/// @note hide assignment operator from interface user
	Win32LastError& operator=(const Win32LastError&);

};

}

#endif